package com.link.core.basic.aop;

import com.link.base.user.model.CoreUser;
import com.link.core.util.BeanTools;
import com.link.core.util.StringUtils;
import com.link.core.util.UserUtil;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import org.apache.ibatis.binding.MapperProxy;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Pointcut;

public class MultiCorpDaoAspect {
   @Pointcut("execution(* com.link.base..dao..*.*(..))")
   public void multiCorpDao() {
   }

   @Around("multiCorpDao()")
   public Object daoPointcut(ProceedingJoinPoint point) throws Throwable {
      CoreUser user = null;

      try {
         user = UserUtil.getUser();
      } catch (Exception var12) {
      }

      if (user == null) {
         return point.proceed();
      } else {
         Long corpId = user.getLoginCorpId();
         if (null == corpId) {
            return point.proceed();
         } else {
            Object target = point.getTarget();
            Object mapperInterface = this.getMapperProxyInterface(target);
            Object corpMapper = this.getCorpMapperBean(mapperInterface, corpId);
            if (corpMapper == null) {
               return point.proceed();
            } else {
               String method = point.getSignature().getName();
               Method[] methods = corpMapper.getClass().getDeclaredMethods();
               Method corpTargetMethod = null;

               for(int i = 0; i < methods.length; ++i) {
                  Method methodItem = methods[i];
                  if (StringUtils.equals(methodItem.getName(), method)) {
                     corpTargetMethod = methodItem;
                     break;
                  }
               }

               if (corpTargetMethod == null) {
                  return point.proceed();
               } else {
                  Object[] args = point.getArgs();
                  Object resultObj = corpTargetMethod.invoke(corpMapper, args);
                  return resultObj;
               }
            }
         }
      }
   }

   private Object getMapperProxyInterface(Object proxy) throws Exception {
      Field h = proxy.getClass().getSuperclass().getDeclaredField("h");
      h.setAccessible(true);
      MapperProxy<?> mapperProxy = (MapperProxy)h.get(proxy);
      Field mapperInterfaceField = mapperProxy.getClass().getDeclaredField("mapperInterface");
      mapperInterfaceField.setAccessible(true);
      Object mapperInterface = mapperInterfaceField.get(mapperProxy);
      return mapperInterface;
   }

   private Object getCorpMapperBean(Object mapperClass, Long corpId) throws Exception {
      String mapperSymbol = "Mapper";
      String className = ((Class)mapperClass).getSimpleName();
      String standardBeanName = StringUtils.firstCharLowerCase(className);
      StringBuilder sb = new StringBuilder(standardBeanName);
      if (standardBeanName.endsWith("Mapper")) {
         sb.insert(sb.lastIndexOf("Mapper"), "C" + corpId);
      }

      String corpBeanName = sb.toString();
      Object corpMapper = BeanTools.getBean(corpBeanName);
      return corpMapper;
   }
}
